# pylint: disable-msg=C0103
# pylint: disable=line-too-long
"""
SentinelAzure: This package is developed for initializing and manipulating Python client objects of Azure Resource Management
"""

# __init__.py
from .azure_loganalytics_helper import *
