# pylint: disable-msg=C0103
# pylint: disable=line-too-long
"""
SentinelPortal: This package is developed for Sentinel notebooks integrating with Sentinel portal
"""

# __init__.py
from .bookmark_helper import Constants, BookmarkProperties, BookmarkModel, BookmarkHelper
