# -------------------------------------------------------------------------

# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
"""
input_error module:
This module include custom exceptions
"""


class InputError(ValueError):

    """ This class provides custom exception for validating input data """
